"use strict";

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _mockery = require("mockery");

var _mockery2 = _interopRequireDefault(_mockery);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MemoryStream = require('memory-stream');


describe("compileClient", function () {
	var mockFs = void 0,
	    writeWebpackCompatibleRoutesFile = void 0;

	beforeAll(function () {
		_mockery2.default.registerSubstitute('./callerDependency', './callerDependency-Mock');
		_mockery2.default.enable({
			useCleanCache: true,
			warnOnUnregistered: false
		});

		writeWebpackCompatibleRoutesFile = require("../../compileClient").writeWebpackCompatibleRoutesFile;
	});

	afterAll(function () {
		_mockery2.default.disable();
	});

	describe("writes client routes file for Webpack", function () {
		var pathStringTests = [{
			title: "apostrophes",
			path: "PathWith'InIt.js"
		}, {
			title: "double quotes",
			path: 'PathWith"InIt.js'
		}, {
			title: "windows style",
			path: 'c:\\Path\\With\\InIt.js'
		}, {
			title: "spaces",
			path: 'Path With Spaces.js'
		}];

		beforeEach(function () {
			mockFs = new MemoryStream();
		});

		afterEach(function () {
			mockFs = null;
		});

		pathStringTests.map(function (test) {
			it("handles file paths with " + test.title, function (finishTest) {
				spyOn(_fs2.default, 'writeFileSync').and.callFake(function (path, data) {
					mockFs.write(data);
				});

				var filePath = test.path;
				var routes = {
					"routes": {
						"Homepage": {
							"path": "/",
							"page": filePath
						}
					}
				};

				writeWebpackCompatibleRoutesFile(routes, ".", _path2.default.normalize("."), null, true, null);

				var coreMiddlewareStringified = (0, _stringify2.default)(require.resolve("react-server-core-middleware"));
				var filePathStringified = (0, _stringify2.default)(filePath);

				// These four strings are important when using multiple platforms or strings with weird characters in them.
				// If we're going to output something to a file and then import that file later, we'd better be darn sure
				// it's all correctly formatted!  Apostrophes, quotes, and windows-style file path characters \ vs / are the worst!
				var filePathRegexStrings = ["var coreJsMiddleware = require(" + coreMiddlewareStringified + ").coreJsMiddleware;", "var coreCssMiddleware = require(" + coreMiddlewareStringified + ").coreCssMiddleware;", "require.ensure(" + filePathStringified + ", function() {", "cb(unwrapEs6Module(require(" + filePathStringified + ")));"];

				var fileData = mockFs.toString();
				filePathRegexStrings.map(function (regex) {
					expect(fileData).toMatch(regex.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&"));
				});

				finishTest();
			});
		});
	});
});