"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

exports.default = start;

var _http = require("http");

var _http2 = _interopRequireDefault(_http);

var _https = require("https");

var _https2 = _interopRequireDefault(_https);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _express = require("express");

var _express2 = _interopRequireDefault(_express);

var _compression = require("compression");

var _compression2 = _interopRequireDefault(_compression);

var _bodyParser = require("body-parser");

var _bodyParser2 = _interopRequireDefault(_bodyParser);

var _helmet = require("helmet");

var _helmet2 = _interopRequireDefault(_helmet);

var _webpackDevServer = require("webpack-dev-server");

var _webpackDevServer2 = _interopRequireDefault(_webpackDevServer);

var _compileClient2 = require("../compileClient");

var _compileClient3 = _interopRequireDefault(_compileClient2);

var _handleCompilationErrors = require("../handleCompilationErrors");

var _handleCompilationErrors2 = _interopRequireDefault(_handleCompilationErrors);

var _reactServer = require("../react-server");

var _reactServer2 = _interopRequireDefault(_reactServer);

var _setupLogging = require("../setupLogging");

var _setupLogging2 = _interopRequireDefault(_setupLogging);

var _logProductionWarnings = require("../logProductionWarnings");

var _logProductionWarnings2 = _interopRequireDefault(_logProductionWarnings);

var _expressState = require("express-state");

var _expressState2 = _interopRequireDefault(_expressState);

var _cookieParser = require("cookie-parser");

var _cookieParser2 = _interopRequireDefault(_cookieParser);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var logger = _reactServer2.default.logging.getLogger({ "name": "react-server-cli.src.commands.start", "color": { "server": 131, "client": "rgb(127,42,42)" } });

// returns a method that can be used to stop the server. the returned method
// returns a promise to indicate when the server is actually stopped.
var serverToStopPromise = function serverToStopPromise(server) {

	var sockets = [];

	// If we're testing then we want to be able to bail out quickly.  Zombie
	// (the test browser) makes keepalive connections to our static asset
	// server, and we don't need to be polite to it when we're tearing down.
	if (process.env.NODE_ENV === "test") {
		// eslint-disable-line no-process-env
		server.on('connection', function (socket) {
			return sockets.push(socket);
		});
	}

	return function () {
		return new _promise2.default(function (resolve, reject) {

			// This will only have anything if we're testing.  See above.
			sockets.forEach(function (socket) {
				return socket.destroy();
			});

			server.on('error', function (e) {
				logger.error('An error was emitted while shutting down the server');
				logger.error(e);
				reject(e);
			});
			server.close(function (e) {
				if (e) {
					logger.error('The server was not started, so it cannot be stopped.');
					logger.error(e);
					reject(e);
					return;
				}
				resolve();
			});
		});
	};
};

// given the server routes file and a port, start a react-server HTML server at
// http://host:port/. returns an object with two properties, started and stop;
// see the default function doc for explanation.
var startHtmlServer = function startHtmlServer(serverRoutes, port, bindIp, httpsOptions, customMiddlewarePath) {
	var server = (0, _express2.default)();
	var httpServer = httpsOptions ? _https2.default.createServer(httpsOptions, server) : _http2.default.createServer(server);
	var middlewareSetup = function middlewareSetup(server, rsMiddleware) {
		server.use((0, _compression2.default)());
		server.use(_bodyParser2.default.urlencoded({ extended: false }));
		server.use(_bodyParser2.default.json());
		server.use((0, _helmet2.default)());
		rsMiddleware();
	};

	return {
		stop: serverToStopPromise(httpServer),
		started: new _promise2.default(function (resolve, reject) {
			logger.info("Starting HTML server...");

			var rsMiddlewareCalled = false;
			var rsMiddleware = function rsMiddleware() {
				rsMiddlewareCalled = true;

				_expressState2.default.extend(server);

				// parse cookies into req.cookies property
				server.use((0, _cookieParser2.default)());

				// sets the namespace that data will be exposed into client-side
				// TODO: express-state doesn't do much for us until we're using a templating library
				server.set('state namespace', '__reactServerState');

				server.use(function (req, res, next) {
					_reactServer2.default.middleware(req, res, next, require(serverRoutes));
				});
			};

			if (customMiddlewarePath) {
				var customMiddlewareDirAb = _path2.default.resolve(process.cwd(), customMiddlewarePath);
				middlewareSetup = require(customMiddlewareDirAb).default;
			}

			middlewareSetup(server, rsMiddleware);

			if (!rsMiddlewareCalled) {
				console.error("Error react-server middleware was never setup in custom middleware function");
				reject("Custom middleware did not setup react-server middleware");
				return;
			}

			httpServer.on('error', function (e) {
				console.error("Error starting up HTML server");
				console.error(e);
				reject(e);
			});
			httpServer.listen(port, bindIp, function (e) {
				if (e) {
					reject(e);
					return;
				}
				logger.info("Started HTML server over " + (httpsOptions ? "HTTPS" : "HTTP") + " on " + bindIp + ":" + port);
				resolve();
			});
		})
	};
};

// given a webpack compiler and a port, compile the JavaScript code to static
// files and start up a web server at http://host:port/ that serves the
// static compiled JavaScript. returns an object with two properties, started and stop;
// see the default function doc for explanation.
var startStaticJsServer = function startStaticJsServer(compiler, port, bindIp, longTermCaching, httpsOptions) {
	var server = (0, _express2.default)();
	var httpServer = httpsOptions ? _https2.default.createServer(httpsOptions, server) : _http2.default.createServer(server);
	return {
		stop: serverToStopPromise(httpServer),
		started: new _promise2.default(function (resolve, reject) {
			compiler.run(function (err, stats) {
				var error = (0, _handleCompilationErrors2.default)(err, stats);
				if (error) {
					reject(error);
					return;
				}

				if (stats) {
					logger.debug("Successfully compiled static JavaScript.");
				}

				// TODO: make this parameterized based on what is returned from compileClient
				server.use('/', (0, _compression2.default)(), _express2.default.static("__clientTemp/build", {
					maxage: longTermCaching ? '365d' : '0s'
				}));
				logger.info("Starting static JavaScript server...");

				httpServer.on('error', function (e) {
					console.error("Error starting up JS server");
					console.error(e);
					reject(e);
				});
				httpServer.listen(port, bindIp, function (e) {
					if (e) {
						reject(e);
						return;
					}

					logger.info("Started static JavaScript server over " + (httpsOptions ? "HTTPS" : "HTTP") + " on " + bindIp + ":" + port);
					resolve();
				});
			});
		})
	};
};

// given a webpack compiler and a port, start a webpack dev server that is ready
// for hot reloading at http://localhost:port/. note that the webpack compiler
// must have been configured correctly for hot reloading. returns an object with
// two properties, started and stop; see the default function doc for explanation.
var startHotLoadJsServer = function startHotLoadJsServer(compiler, port, bindIp, longTermCaching, httpsOptions) {
	logger.info("Starting hot reload JavaScript server...");
	var compiledPromise = new _promise2.default(function (resolve) {
		return compiler.plugin("done", function () {
			return resolve();
		});
	});
	var jsServer = new _webpackDevServer2.default(compiler, {
		noInfo: true,
		hot: true,
		headers: { 'Access-Control-Allow-Origin': '*' },
		https: !!httpsOptions,
		key: httpsOptions ? httpsOptions.key : undefined,
		cert: httpsOptions ? httpsOptions.cert : undefined,
		ca: httpsOptions ? httpsOptions.ca : undefined
	});
	var serverStartedPromise = new _promise2.default(function (resolve, reject) {
		jsServer.listen(port, bindIp, function (e) {
			if (e) {
				reject(e);
				return;
			}
			resolve();
		});
	});
	return {
		stop: serverToStopPromise(jsServer),
		started: _promise2.default.all([compiledPromise, serverStartedPromise]).then(function () {
			return logger.info("Started hot reload JavaScript server over " + (httpsOptions ? "HTTPS" : "HTTP") + " on " + bindIp + ":" + port);
		})
	};
};

// for when you need to run the JavaScript compiler (in order to get the chunk file
// names for the server routes file) but don't really want to actually up a JavaScript
// server. Supports the same signature as startStaticJsServer and startHotLoadJsServer,
// returning the same {stop, started} object.
var startDummyJsServer = function startDummyJsServer(compiler /*, port, longTermCaching, httpsOptions*/) {
	return {
		stop: function stop() {
			return _promise2.default.resolve();
		},
		started: new _promise2.default(function (resolve, reject) {
			return compiler.run(function (err, stats) {
				// even though we aren't using the compiled code (we're pointing at jsUrl),
				// we still need to run the compilation to get the chunk file names.
				try {
					(0, _handleCompilationErrors2.default)(err, stats);
				} catch (e) {
					logger.emergency("Failed to compile the local code.", e.stack);
					reject(e);
					return;
				}
				resolve();
			});
		})
	};
};

// if used to start a server, returns an object with two properties, started and
// stop. started is a promise that resolves when all necessary servers have been
// started. stop is a method to stop all servers. It takes no arguments and
// returns a promise that resolves when the server has stopped.
function start(options) {
	(0, _setupLogging2.default)(options);
	(0, _logProductionWarnings2.default)(options);

	var port = options.port,
	    bindIp = options.bindIp,
	    jsPort = options.jsPort,
	    hot = options.hot,
	    jsUrl = options.jsUrl,
	    httpsOptions = options.httpsOptions,
	    longTermCaching = options.longTermCaching,
	    customMiddlewarePath = options.customMiddlewarePath,
	    compileOnStartup = options.compileOnStartup;


	var routesFileLocation = _path2.default.join(process.cwd(), '__clientTemp/routes_server.js');
	var serverRoutes = void 0,
	    compiler = void 0;

	var startServers = function startServers() {
		// if jsUrl is set, we need to run the compiler, but we don't want to start a JS
		// server.
		var startJsServer = startDummyJsServer;

		if ((hot === false || hot === "false") && (compileOnStartup === false || compileOnStartup === "false")) {
			serverRoutes = new _promise2.default(function (resolve, reject) {
				_fs2.default.access(routesFileLocation, _fs2.default.constants.R_OK, function (err) {
					if (err) {
						reject("You must manually compile your application when compileOnStartup is set to false.");
					} else {
						// We need to replace the promise returned by the compiler with an already-resolved promise with the path
						// of the compiled routes file.
						resolve(routesFileLocation);
					}
				});
			});

			// mock the compiler object used by the various JS servers so that the compiler.run function always succeeds.
			// This will allow the JS servers to work properly, thinking that the compiler actually ran.
			compiler = {};
			compiler.run = function (cb) {
				cb(null, null);
			};
		} else {
			var _compileClient = (0, _compileClient3.default)(options);
			// ES6 destructuring without a preceding `let` or `const` results in a syntax error.  Therefore, the below
			// statement must be wrapped in parentheses to work properly.
			// http://exploringjs.com/es6/ch_destructuring.html#sec_leading-curly-brace-destructuring


			serverRoutes = _compileClient.serverRoutes;
			compiler = _compileClient.compiler;
		}

		if (!jsUrl) {
			// if jsUrl is not set, we need to start up a JS server, either hot load
			// or static.
			startJsServer = hot ? startHotLoadJsServer : startStaticJsServer;
		}

		logger.notice("Starting servers...");

		var jsServer = startJsServer(compiler, jsPort, bindIp, longTermCaching, httpsOptions);
		var htmlServerPromise = serverRoutes.then(function (serverRoutesFile) {
			return startHtmlServer(serverRoutesFile, port, bindIp, httpsOptions, customMiddlewarePath);
		}).catch(function (e) {
			throw e;
		});

		return {
			stop: function stop() {
				return _promise2.default.all([jsServer.stop(), htmlServerPromise.then(function (server) {
					return server.stop();
				})]);
			},
			started: _promise2.default.all([jsServer.started, htmlServerPromise.then(function (server) {
				return server.started;
			})]).catch(function (e) {
				logger.error(e);throw e;
			}).then(function () {
				return logger.notice("Ready for requests on " + bindIp + ":" + port + ".");
			})
		};
	};

	return startServers();
}
module.exports = exports["default"];