"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _getIterator2 = require("babel-runtime/core-js/get-iterator");

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

exports.writeWebpackCompatibleRoutesFile = writeWebpackCompatibleRoutesFile;

var _webpack = require("webpack");

var _webpack2 = _interopRequireDefault(_webpack);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _mkdirp = require("mkdirp");

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _extractTextWebpackPlugin = require("extract-text-webpack-plugin");

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _chunkManifestWebpackPlugin = require("chunk-manifest-webpack-plugin");

var _chunkManifestWebpackPlugin2 = _interopRequireDefault(_chunkManifestWebpackPlugin);

var _webpackStatsPlugin = require("webpack-stats-plugin");

var _webpackStatsPlugin2 = _interopRequireDefault(_webpackStatsPlugin);

var _callerDependency = require("./callerDependency");

var _callerDependency2 = _interopRequireDefault(_callerDependency);

var _serverSideHotModuleReload = require("./serverSideHotModuleReload");

var _serverSideHotModuleReload2 = _interopRequireDefault(_serverSideHotModuleReload);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// commented out to please eslint, but re-add if logging is needed in this file.
// import {logging} from "react-server"
// const logger = logging.getLogger({"name":"react-server-cli.src.compileClient","color":{"server":135,"client":"rgb(127,42,212)"}});

// compiles the routes file for browser clients using webpack.
// returns a tuple of { compiler, serverRoutes }. compiler is a webpack compiler
// that is ready to have run called, and serverRoutes is a promise that resolve to
// a path to the transpiled server routes file path. The promise only resolves
// once the compiler has been run. The file path returned from the promise
// can be required and passed in to reactServer.middleware().
// TODO: add options for sourcemaps.
exports.default = function () {
	var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
	var routes = opts.routes,
	    webpackConfig = opts.webpackConfig,
	    _opts$workingDir = opts.workingDir,
	    workingDir = _opts$workingDir === undefined ? "./__clientTemp" : _opts$workingDir,
	    _opts$routesDir = opts.routesDir,
	    routesDir = _opts$routesDir === undefined ? "." : _opts$routesDir,
	    _opts$outputDir = opts.outputDir,
	    outputDir = _opts$outputDir === undefined ? workingDir + "/build" : _opts$outputDir,
	    _opts$outputUrl = opts.outputUrl,
	    outputUrl = _opts$outputUrl === undefined ? "/static/" : _opts$outputUrl,
	    _opts$hot = opts.hot,
	    hot = _opts$hot === undefined ? true : _opts$hot,
	    _opts$minify = opts.minify,
	    minify = _opts$minify === undefined ? false : _opts$minify,
	    _opts$stats = opts.stats,
	    stats = _opts$stats === undefined ? false : _opts$stats,
	    _opts$longTermCaching = opts.longTermCaching,
	    longTermCaching = _opts$longTermCaching === undefined ? false : _opts$longTermCaching;

	// support legacy webpack configuration name

	var webpackConfiguration = webpackConfig || opts['webpack-config'];
	if (longTermCaching && hot) {
		// chunk hashes can't be used in hot mode, so we can't use long-term caching
		// and hot mode at the same time.
		throw new Error("Hot reload cannot be used with long-term caching. Please disable either long-term caching or hot reload.");
	}

	var webpackConfigFunc = function webpackConfigFunc(data) {
		return data;
	};
	if (webpackConfiguration) {
		var webpackDirAbsolute = _path2.default.resolve(process.cwd(), webpackConfiguration);
		var userWebpackConfigFunc = require(webpackDirAbsolute);
		webpackConfigFunc = userWebpackConfigFunc.default;
	}

	var workingDirAbsolute = _path2.default.resolve(process.cwd(), workingDir);
	_mkdirp2.default.sync(workingDirAbsolute);
	var outputDirAbsolute = _path2.default.resolve(process.cwd(), outputDir);
	_mkdirp2.default.sync(outputDirAbsolute);

	var routesDirAbsolute = _path2.default.resolve(process.cwd(), routesDir);

	// for each route, let's create an entrypoint file that includes the page file and the routes file
	var bootstrapFile = writeClientBootstrapFile(workingDirAbsolute, opts);

	// This is required because WebpackDevServer uses Node's url module to parse the URL.  That module does not
	// support protocol-relative URLs, therefore doesn't actually parse the URL properly.  We can "fake" out the
	// url module by specifying http://.  Don't worry, WebpackDevServer will actually use window.location.protocol
	// instead of what we hardcode.
	// Node issue: https://github.com/nodejs/node-v0.x-archive/issues/9123
	// WebpackDevServer overriding protocol: https://github.com/webpack/webpack-dev-server/blob/master/client/index.js#L123-L129
	var webpackDevServerClientUrl = outputUrl;
	if (/^\/\//.test(outputUrl)) {
		// this is a protocol relative URL like '//0.0.0.0'
		webpackDevServerClientUrl = "http:" + outputUrl;
	}
	var entrypointBase = hot ? [require.resolve("webpack-dev-server/client") + "?" + webpackDevServerClientUrl, require.resolve("webpack/hot/only-dev-server")] : [];
	var entrypoints = {};
	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = (0, _getIterator3.default)((0, _keys2.default)(routes.routes)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var routeName = _step.value;

			var route = routes.routes[routeName];
			var formats = normalizeRoutesPage(route.page);
			var _iteratorNormalCompletion2 = true;
			var _didIteratorError2 = false;
			var _iteratorError2 = undefined;

			try {
				for (var _iterator2 = (0, _getIterator3.default)((0, _keys2.default)(formats)), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
					var format = _step2.value;

					var absolutePathToPage = require.resolve(_path2.default.resolve(routesDirAbsolute, formats[format]));

					entrypoints["" + routeName + (format !== "default" ? "-" + format : "")] = [].concat(entrypointBase, [bootstrapFile, absolutePathToPage]);
				}
			} catch (err) {
				_didIteratorError2 = true;
				_iteratorError2 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion2 && _iterator2.return) {
						_iterator2.return();
					}
				} finally {
					if (_didIteratorError2) {
						throw _iteratorError2;
					}
				}
			}
		}

		// now rewrite the routes file out in a webpack-compatible way.
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}

	writeWebpackCompatibleRoutesFile(routes, routesDir, workingDirAbsolute, null, true);

	// finally, let's pack this up with webpack.
	var compiler = (0, _webpack2.default)(webpackConfigFunc(packageCodeForBrowser(entrypoints, outputDirAbsolute, outputUrl, hot, minify, longTermCaching, stats)));

	var serverRoutes = new _promise2.default(function (resolve) {
		compiler.plugin("done", function (stats) {
			var manifest = statsToManifest(stats);
			_fs2.default.writeFileSync(_path2.default.join(outputDir, "manifest.json"), (0, _stringify2.default)(manifest));

			// this file is generated by the build in non-hot mode, but we don't need
			// it (we got the data ourselves from stats in statsToManifest()).
			if (!hot) {
				_fs2.default.unlinkSync(_path2.default.join(outputDir, "chunk-manifest.json"));
			}

			var routesFilePath = writeWebpackCompatibleRoutesFile(routes, routesDir, workingDirAbsolute, outputUrl, false, manifest);

			if (hot) {
				(0, _serverSideHotModuleReload2.default)(stats);
			}

			resolve(routesFilePath);
		});
	});

	return {
		serverRoutes: serverRoutes,
		compiler: compiler
	};
};

// takes in the stats object from a successful compilation and returns a manifest
// object that characterizes the results of the compilation for CSS/JS loading
// and integrity checking. the manifest has the following entries:
//   jsChunksByName: an object that maps chunk names to their JS entrypoint file.
//   cssChunksByName: an object that maps chunk names to their CSS file. Note that
//     not all named chunks necessarily have a CSS file, so not all names will
//     show up as a key in this object.
//   jsChunksById: an object that maps chunk ids to their JS entrypoint file.
//   hash: the overall hash of the build, which can be used to check integrity
//     with prebuilt sources.


function statsToManifest(stats) {
	var jsChunksByName = {};
	var cssChunksByName = {};
	var jsChunksById = {};
	var _iteratorNormalCompletion3 = true;
	var _didIteratorError3 = false;
	var _iteratorError3 = undefined;

	try {
		for (var _iterator3 = (0, _getIterator3.default)(stats.compilation.chunks), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
			var chunk = _step3.value;

			if (chunk.name) {
				jsChunksByName[chunk.name] = chunk.files[0];

				for (var i = 1; i < chunk.files.length; i++) {
					if (/\.css$/.test(chunk.files[i])) {
						// the CSS file is probably last file in the array, unless there have been hot updates for the JS
						// which show up in the array prior to the CSS file.
						cssChunksByName[chunk.name] = chunk.files[i];
						break;
					}
				}
			}
			jsChunksById[chunk.id] = chunk.files[0];
		}
	} catch (err) {
		_didIteratorError3 = true;
		_iteratorError3 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion3 && _iterator3.return) {
				_iterator3.return();
			}
		} finally {
			if (_didIteratorError3) {
				throw _iteratorError3;
			}
		}
	}

	return {
		jsChunksByName: jsChunksByName,
		jsChunksById: jsChunksById,
		cssChunksByName: cssChunksByName,
		hash: stats.hash
	};
}

function packageCodeForBrowser(entrypoints, outputDir, outputUrl, hot, minify, longTermCaching, stats) {
	var NonCachingExtractTextLoader = _path2.default.join(__dirname, "./NonCachingExtractTextLoader");
	var extractTextLoader = require.resolve(NonCachingExtractTextLoader) + "?{remove:true}!css-loader";
	var webpackConfig = {
		entry: entrypoints,
		output: {
			path: outputDir,
			// other than hot mode, the public path is set at runtime.
			publicPath: hot ? outputUrl : undefined,
			filename: "[name]" + (longTermCaching ? ".[chunkhash]" : "") + ".bundle.js",
			chunkFilename: "[id]" + (longTermCaching ? ".[chunkhash]" : "") + ".bundle.js"
		},
		module: {
			loaders: [{
				test: /\.jsx?$/,
				loader: "babel",
				exclude: /node_modules/
			}, {
				test: /\.css$/,
				loader: extractTextLoader
			}, {
				test: /\.(eot|woff|woff2|ttf|ttc|png|svg|jpg|jpeg|gif|cgm|tiff|webp|bmp|ico)$/i,
				loader: "file",
				exclude: /node_modules/
			}, {
				test: /\.json/,
				loader: "json"
				// exclude: /node_modules/,
			}, {
				test: /\.less/,
				loader: extractTextLoader + "!less-loader",
				exclude: /node_modules/
			}, {
				test: /\.s(a|c)ss$/,
				loader: extractTextLoader + "!sass-loader",
				exclude: /node_modules/
			}, {
				test: /\.md/,
				loader: "raw",
				exclude: /node_modules/
			}]
		},
		resolve: {
			alias: {
				// These need to be singletons.
				"react": (0, _callerDependency2.default)("react"),
				"react-server": (0, _callerDependency2.default)("react-server")
			}
		},
		resolveLoader: {
			root: [_path2.default.resolve(_path2.default.join(_path2.default.dirname(require.resolve("webpack")), "../.."))]
		},
		plugins: [new _chunkManifestWebpackPlugin2.default({
			filename: "chunk-manifest.json",
			manifestVariable: "webpackManifest"
		}), new _extractTextWebpackPlugin2.default("[name]" + (longTermCaching ? ".[chunkhash]" : "") + ".css"), new _webpack2.default.optimize.CommonsChunkPlugin({
			name: "common"
		})]
	};

	if (stats) {
		webpackConfig.plugins.push(new _webpackStatsPlugin2.default.StatsWriterPlugin({
			fields: ["assets", "assetsByChunkName", "chunks", "errors", "warnings", "version", "hash", "time", "filteredModules", "children", "modules"]
		}));
	}

	if (minify) {
		webpackConfig.plugins = [].concat((0, _toConsumableArray3.default)(webpackConfig.plugins), [new _webpack2.default.DefinePlugin({
			'process.env': { NODE_ENV: '"production"' }
		}),
		// TODO: should this be done as babel plugin?
		new _webpack2.default.optimize.UglifyJsPlugin()]);
	} else {
		webpackConfig.plugins.push(new _webpack2.default.SourceMapDevToolPlugin());
	}

	if (hot) {
		webpackConfig.module.loaders = [{
			test: /\.jsx?$/,
			loader: "react-hot-loader/webpack",
			exclude: /node_modules/
		}].concat((0, _toConsumableArray3.default)(webpackConfig.module.loaders));
		webpackConfig.plugins = [].concat((0, _toConsumableArray3.default)(webpackConfig.plugins), [new _webpack2.default.HotModuleReplacementPlugin(), new _webpack2.default.NoErrorsPlugin()]);
	}

	if (longTermCaching) {
		webpackConfig.plugins = [new _webpack2.default.optimize.OccurenceOrderPlugin()].concat((0, _toConsumableArray3.default)(webpackConfig.plugins));
	}

	return webpackConfig;
}

// writes out a routes file that can be used at runtime.
function writeWebpackCompatibleRoutesFile(routes, routesDir, workingDirAbsolute, staticUrl, isClient, manifest) {
	var routesOutput = [];

	var coreMiddleware = (0, _stringify2.default)(require.resolve("react-server-core-middleware"));
	var existingMiddleware = routes.middleware ? routes.middleware.map(function (middlewareRelativePath) {
		return "unwrapEs6Module(require(" + (0, _stringify2.default)(_path2.default.relative(workingDirAbsolute, _path2.default.resolve(routesDir, middlewareRelativePath))) + "))";
	}) : [];
	routesOutput.push("\nvar manifest = " + (manifest ? (0, _stringify2.default)(manifest) : "undefined") + ";\nfunction unwrapEs6Module(module) { return module.__esModule ? module.default : module }\nvar coreJsMiddleware = require(" + coreMiddleware + ").coreJsMiddleware;\nvar coreCssMiddleware = require(" + coreMiddleware + ").coreCssMiddleware;\nmodule.exports = {\n\tmiddleware:[\n\t\tcoreJsMiddleware(" + (0, _stringify2.default)(staticUrl) + ", manifest),\n\t\tcoreCssMiddleware(" + (0, _stringify2.default)(staticUrl) + ", manifest),\n\t\t" + existingMiddleware.join(",") + "\n\t],\n\troutes:{");

	var _iteratorNormalCompletion4 = true;
	var _didIteratorError4 = false;
	var _iteratorError4 = undefined;

	try {
		for (var _iterator4 = (0, _getIterator3.default)((0, _keys2.default)(routes.routes)), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
			var routeName = _step4.value;

			var route = routes.routes[routeName];

			// On the line below specifying 'method', if the route doesn't have a method, we'll set it to `undefined` so that
			// routr passes through and matches any method
			// https://github.com/yahoo/routr/blob/v2.1.0/lib/router.js#L49-L57
			var method = route.method;

			// Safely check for an empty object, array, or string and specifically set it to 'undefined'
			if (method === undefined || method === null || method === {} || method.length === 0) {
				method = undefined; // 'undefined' is the value that routr needs to accept any method
			}

			routesOutput.push("\n\t\t" + routeName + ": {\n\t\t\tpath: " + (0, _stringify2.default)(route.path) + ",\n\t\t\tmethod: " + (0, _stringify2.default)(method) + ",");

			var formats = normalizeRoutesPage(route.page);
			routesOutput.push("\n\t\t\tpage: {");
			var _iteratorNormalCompletion5 = true;
			var _didIteratorError5 = false;
			var _iteratorError5 = undefined;

			try {
				for (var _iterator5 = (0, _getIterator3.default)((0, _keys2.default)(formats)), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
					var format = _step5.value;

					var formatModule = formats[format];
					var relativePathToPage = (0, _stringify2.default)(_path2.default.relative(workingDirAbsolute, _path2.default.resolve(routesDir, formatModule)));
					routesOutput.push("\n\t\t\t\t" + format + ": function() {\n\t\t\t\t\treturn {\n\t\t\t\t\t\tdone: function(cb) {");
					if (isClient) {
						routesOutput.push("\n\t\t\t\t\t\t\trequire.ensure(" + relativePathToPage + ", function() {\n\t\t\t\t\t\t\t\tcb(unwrapEs6Module(require(" + relativePathToPage + ")));\n\t\t\t\t\t\t\t});");
					} else {
						routesOutput.push("\n\t\t\t\t\t\t\ttry {\n\t\t\t\t\t\t\t\tcb(unwrapEs6Module(require(" + relativePathToPage + ")));\n\t\t\t\t\t\t\t} catch (e) {\n\t\t\t\t\t\t\t\tconsole.error('Failed to load page at " + relativePathToPage + "', e.stack);\n\t\t\t\t\t\t\t}");
					}
					routesOutput.push("\n\t\t\t\t\t\t}\n\t\t\t\t\t};\n\t\t\t\t},");
				}
			} catch (err) {
				_didIteratorError5 = true;
				_iteratorError5 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion5 && _iterator5.return) {
						_iterator5.return();
					}
				} finally {
					if (_didIteratorError5) {
						throw _iteratorError5;
					}
				}
			}

			routesOutput.push("\n\t\t\t},\n\t\t},");
		}
	} catch (err) {
		_didIteratorError4 = true;
		_iteratorError4 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion4 && _iterator4.return) {
				_iterator4.return();
			}
		} finally {
			if (_didIteratorError4) {
				throw _iteratorError4;
			}
		}
	}

	routesOutput.push("\n\t}\n};");

	var routesContent = routesOutput.join("");
	// make a unique file name so that when it is required, there are no collisions
	// in the module loader between different invocations.
	var routesFilePath = workingDirAbsolute + "/routes_" + (isClient ? "client" : "server") + ".js";
	_fs2.default.writeFileSync(routesFilePath, routesContent);

	return routesFilePath;
}

// the page value for routes.routes["SomeRoute"] can either be a string for the default
// module name or an object mapping format names to module names. This method normalizes
// the value to an object.
function normalizeRoutesPage(page) {
	if (typeof page === "string") {
		return { default: page };
	}
	return page;
}

// writes out a bootstrap file for the client which in turn includes the client
// routes file. note that outputDir must be the same directory as the client routes
// file, which must be named "routes_client".
function writeClientBootstrapFile(outputDir, opts) {
	var outputFile = outputDir + "/entry.js";
	_fs2.default.writeFileSync(outputFile, "\n\t\tif (typeof window !== \"undefined\") {\n\t\t\twindow.__setReactServerBase = function(path) {\n\t\t\t\t// according to http://webpack.github.io/docs/configuration.html#output-publicpath\n\t\t\t\t// we should never set __webpack_public_path__ when hot module replacement is on.\n\t\t\t\tif (!module.hot) {\n\t\t\t\t\t__webpack_public_path__ = path;\n\t\t\t\t}\n\t\t\t\twindow.__reactServerBase = path;\n\t\t\t}\n\t\t}\n\t\tvar reactServer = require(\"react-server\");\n\t\twindow.rfBootstrap = function() {\n\t\t\treactServer.logging.setLevel('main',  " + (0, _stringify2.default)(opts.logLevel) + ");\n\t\t\treactServer.logging.setLevel('time',  " + (0, _stringify2.default)(opts.timingLogLevel) + ");\n\t\t\treactServer.logging.setLevel('gauge', " + (0, _stringify2.default)(opts.gaugeLogLevel) + ");\n\t\t\tnew reactServer.ClientController({routes: require(\"./routes_client\")}).init();\n\t\t}");
	return outputFile;
}