"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// implements the search algorithm for finding react-server config files. see
// the README.md for a description.
var REACT_SERVER_RC = ".reactserverrc";
var PACKAGE_JSON = "package.json";

// returns an options object that represents the **first** config file found in the
// search path. if none are found, returns null.
// as described in the README, this method starts at dir, and for each directory
// looks first for a .reactserverrc file, then for a "reactServer" section of
// the package.json. if it finds neither, it goes up a directory and looks again.
// it returns the contents of the first config file found; it never merges multiple
// configurations.

exports.default = function () {
	var dir = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : process.cwd();

	do {
		var reactServerRc = null;
		try {
			// readFileSync throws if the file doesn't exist.
			reactServerRc = _fs2.default.readFileSync(_path2.default.join(dir, REACT_SERVER_RC));
		} catch (e) {} //eslint-disable-line no-empty
		if (reactServerRc) {
			return JSON.parse(reactServerRc);
		}

		var packageJson = null;
		try {
			packageJson = _fs2.default.readFileSync(_path2.default.join(dir, PACKAGE_JSON));
		} catch (e) {} //eslint-disable-line no-empty

		if (packageJson) {
			var parsedPackageJson = JSON.parse(packageJson);
			if (parsedPackageJson.reactServer) {
				return parsedPackageJson.reactServer;
			}
		}
	} while (dir !== (dir = _path2.default.dirname(dir)));

	return null;
};

module.exports = exports["default"];