"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = handleCompilationErrors;

var _reactServer = require("./react-server");

var logger = _reactServer.logging.getLogger({ "name": "react-server-cli.src.handleCompilationErrors", "color": { "server": 135, "client": "rgb(127,42,212)" } });

// takes in the err and stats object returned by a webpack compilation and returns
// an error object if something serious happened, or null if things are ok.
function handleCompilationErrors(err, stats) {
	if (err) {
		logger.error("Error during webpack build.");
		logger.error(err);
		return new Error(err);
		// TODO: inspect stats to see if there are errors -sra.
	} else if (stats && stats.hasErrors()) {
		console.error("There were errors in the JavaScript compilation.");
		stats.toJson().errors.forEach(function (error) {
			console.error(error);
		});
		return new Error("There were errors in the JavaScript compilation.");
	} else if (stats && stats.hasWarnings()) {
		logger.warning("There were warnings in the JavaScript compilation. Note that this is normal if you are minifying your code.");
		// for now, don't enumerate warnings; they are absolutely useless in minification mode.
		// TODO: handle this more intelligently, perhaps with a --reportwarnings flag or with different
		// behavior based on whether or not --minify is set.
	}
	return null;
}
module.exports = exports["default"];