"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// takes in an array of options objects for `startServer.js`, and returns a merged
// version of them. it essentially acts like Object.assign(), so options objects
// later in the argument list override the properties of ones earlier in the list.
// like Object.assign, the merging is done on a property-by-property basis;
// they are not merged deeply.
// the only thing that makes this not exactly Object.assign is that options objects
// can have an env attribute that contains overrides for different values for NODE_ENV.
var merge = function merge(options) {
	for (var _len = arguments.length, optionsToBeMerged = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
		optionsToBeMerged[_key - 1] = arguments[_key];
	}

	var optionsCopy = (0, _assign2.default)({}, options);

	// first merge in any overrides based on the current node environment.
	if (optionsCopy.env) {
		var env = optionsCopy.env;
		delete optionsCopy.env;
		// if there's an override for the current NODE_ENV, let's use it.
		if (env[process.env.NODE_ENV]) {
			//eslint-disable-line no-process-env
			optionsCopy = merge(optionsCopy, env[process.env.NODE_ENV]); //eslint-disable-line no-process-env
		}
	}

	// now merge in the rest of the options objects.
	if (optionsToBeMerged.length > 0) {
		return (0, _assign2.default)(optionsCopy, merge.apply(undefined, optionsToBeMerged));
	} else {
		return optionsCopy;
	}
};

exports.default = merge;
module.exports = exports["default"];