"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = run;

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _chalk = require("chalk");

var _chalk2 = _interopRequireDefault(_chalk);

var _mergeOptions = require("./mergeOptions");

var _mergeOptions2 = _interopRequireDefault(_mergeOptions);

var _findOptionsInFiles = require("./findOptionsInFiles");

var _findOptionsInFiles2 = _interopRequireDefault(_findOptionsInFiles);

var _defaultOptions = require("./defaultOptions");

var _defaultOptions2 = _interopRequireDefault(_defaultOptions);

var _ConfigurationError = require("./ConfigurationError");

var _ConfigurationError2 = _interopRequireDefault(_ConfigurationError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable consistent-return */
function run() {
	var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};


	// for the option properties that weren't sent in, look for a config file
	// (either .reactserverrc or a reactServer section in a package.json). for
	// options neither passed in nor in a config file, use the defaults.
	options = (0, _mergeOptions2.default)(_defaultOptions2.default, (0, _findOptionsInFiles2.default)() || {}, options);

	var _options = options,
	    routesFile = _options.routesFile,
	    jsUrl = _options.jsUrl,
	    jsPort = _options.jsPort,
	    host = _options.host;


	options.routesPath = _path2.default.resolve(process.cwd(), routesFile);
	options.routesDir = _path2.default.dirname(options.routesPath);

	// No routes file available when performing init command
	if (options.command !== 'init') {
		try {
			options.routes = require(options.routesPath);
		} catch (e) {
			if (e.code === 'MODULE_NOT_FOUND') {
				console.error(_chalk2.default.red("Failed to load routes file at " + options.routesPath));
			}

			throw e;
		}
	}

	// The non-jsUrl created output URL should be as relative as possible.  '//' ensures that we use the same
	// scheme as the browser, which is important when using a fronting server that terminates HTTPS connections and
	// proxies requests to react-server over HTTP.  Another note, is that `host` should be `0.0.0.0` to make it relative
	// to the hostname being used.
	options.outputUrl = jsUrl || "//" + host + ":" + jsPort + "/";

	try {
		return require("./" + _path2.default.join("commands", options.command))(options);
	} catch (e) {
		if (e instanceof _ConfigurationError2.default) {
			console.error(_chalk2.default.red(e.message));
		} else {
			throw e;
		}
	}
}
module.exports = exports["default"];